/*
 * $Id: //depot/prod/newport_news/Radamant.1/src/java/com/netapp/common/cli/input/InputPrompt.java#1 $
 * 
 * Copyright (c) 2007 Network Appliance, Inc. 
 * All rights reserved.
 */
package com.netapp.common.cli.input;

import java.io.IOException;
import java.io.InputStream;
import java.io.PrintStream;
import java.io.PrintWriter;

import jline.ConsoleReader;

/**
 * Display a prompt to the user. Block reading for the next user entered line break character.
 * Return the input string to the caller.
 * 
 * Typically, this class is called using: <br />
 * 
 * <code>
 *   InputPrompt prompt = new InputPrompt(System.in, System.out);
 * </code>
 * 
 * @author joshb
 *
 */
public class InputPrompt {
    private InputStream input;
    private PrintStream output;

    public InputPrompt(InputStream input, PrintStream output) {
        setInput(input);
        setOutput(output);
    }

    /**
     * Display the prompt to the user. Read the next line of input and return it.
     * 
     * @param prompt
     * @return line of text read from the console
     * @throws IOException 
     */
    public String prompt(String prompt) throws IOException {
        ConsoleReader reader = new ConsoleReader(getInput(), new PrintWriter(getOutput()));
        
        if (null == getReplacementCharacter()) {
            return reader.readLine(prompt);
        }
        else {
            return reader.readLine(prompt, getReplacementCharacter());
        }
    }

    /**
     * Character used to replace the current character on the screen.
     * 
     * @return replacement character
     */
    protected Character getReplacementCharacter() {
        return null;
    }

    protected InputStream getInput() {
        return input;
    }

    protected void setInput(InputStream input) {
        this.input = input;
    }

    protected PrintStream getOutput() {
        return output;
    }

    protected void setOutput(PrintStream output) {
        this.output = output;
    }
}
