/*
 * $Id: //depot/prod/newport_news/Radamant.1/src/java/com/netapp/common/credentials/service/CredentialService.java#1 $
 * 
 * Copyright (c) 2007 Network Appliance, Inc. 
 * All rights reserved.
 */
package com.netapp.common.credentials.service;

import java.util.List;

import javax.jws.WebMethod;
import javax.jws.WebParam;
import javax.jws.WebResult;
import javax.jws.WebService;

import com.netapp.common.credentials.Credential;
import com.netapp.common.credentials.CredentialKey;
import com.netapp.common.flow.api.OperationResult;

/**
 * Interface for managing credentials over SOAP. As this interface is for SOAP, any
 * password that is passed in or out must be encrypted properly. Incoming passwords must
 * be encrypted using a public key. Outgoing passwords must be encrypted using a private key.
 * 
 * @author joshb
 *
 */
@WebService (name = "CredentialService", targetNamespace = "http://www.netapp.com/smai/Credential")
public interface CredentialService {

    /**
     * Attempt to add a new credential into storage. Messages and status are sent back in OperationResult.
     * 
     * @param credential
     * @param type
     * @return request status
     */
    @WebMethod(operationName = "addCredential", action = "urn:addCredential")
    @WebResult(name = "result")
    OperationResult addCredential(@WebParam(name = "credential") Credential credential,
        @WebParam(name = "type") String type);
    
    /**
     * Retreive a credential from storage.
     * 
     * @param key
     * @param type
     * @return request status
     */
    @WebMethod(operationName = "getCredential", action = "urn:getCredential")
    @WebResult(name = "credential")
    Credential getCredential(@WebParam(name = "credentialKey") CredentialKey key, 
        @WebParam(name = "type") String type);
    
    /**
     * Remove the credential from storage. Messages and status are sent back in OperationResult.
     * 
     * @param key
     * @param type
     * @return request status
     */
    @WebMethod(operationName = "removeCredential", action = "urn:removeCredential")
    @WebResult(name = "result")
    OperationResult removeCredential(@WebParam(name = "credentialKey") CredentialKey key, 
        @WebParam(name = "type") String type);

    /**
     * Retrieve the stored credentials. If type is null, return all credentials. Otherwise, return
     * only those credentials that match the type filter. If an internal error occurs, return null.
     * 
     * @param type
     * @return request status
     */
    @WebMethod(operationName = "listCredentials", action = "urn:listCredentials")
    @WebResult(name = "credentials")
    List<Credential> listCredentials(@WebParam(name = "type") String type);
    
    /**
     * Request the current release interface version of the service.  This version reflects a specific 
     * release point of the feature/functional baseline.
     * 
     * @return String
     */
    @WebMethod(operationName = "releaseVersion", action = "urn:releaseVersion")
    @WebResult(name = "releaseVersion")
    String releaseVersion();
    
    /**
     * Request the current semantic interface version of the service.  This version reflects a behavioral 
     * level version of the Service and would be revisioned not solely based on the API syntax level changes, 
     * but also where internal behavioral changes may impact the client side utilization of the service 
     * between those versions.
     * 
     * @return String
     */
    @WebMethod(operationName = "interfaceVersion", action = "urn:interfaceVersion")
    @WebResult(name = "interfaceVersion")
    String interfaceVersion();
}
