/*
 * $Id: //depot/prod/newport_news/Radamant.1/src/java/com/netapp/common/flow/Task.java#1 $
 * 
 * Copyright (c) 2007 Network Appliance, Inc.
 * All rights reserved.
 */
package com.netapp.common.flow;

/**
 * Tasks are the basic building blocks of a workflow operation.
 * Implementing classes should meet the following contract:
 * <ul>
 *   <li>A zero-argument constructor
 *   <li>Task implementation in the {@link #execute()} method.
 *   <li>Java Bean setter methods for any exposed input parameters
 *   <li>Java Bean getter methods for any exposed output parameters
 * </ul>
 * 
 * Tasks will be executed as follows:
 * <ul>
 *   <li>A new task object is constructed
 *   <li>Any inputs specified in the operation configuration will be set via the
 *       setter methods from the context registers.
 *   <li>{@link #execute()} is called. 
 *   <li>If a {@link RetryableException} is thrown during execute, the operation
 *         will be suspended and restarted at a later time from the last checkpoint.
 *   <li>If any other exception is thrown while executing a task on the normal task list,
 *         the operation will be aborted starting at the last abort checkpoint, or fail the
 *         operation as INCOMPLETE if already executing the abort list.
 *   <li>Otherwise, any outputs specified in the operation configuration will be set
 *       via the getter methods into the context registers and execution will continue to the
 *       next checkpoint.
 * </ul>
 * 
 * Tasks should be written with the ability to restart from the previous checkpoint repeatedly
 * without failing if parts of the task were completed before the next checkpoint was reached.
 * For example, a task to create a volume should not fail if the volume requested already exists.
 * If the operation semantics dictate creating an existing volume as a failure case, a precondition
 * should check the volume does not exist, prior to a checkpoint allowing the actual volume create
 * task to restart multiple times after any point within the task has been executed.
 * 
 * @author sunday
 * @version $Revision: #1 $ $Date: 2010/08/20 $
 */
public interface Task {
    /**
     * Execute the task. See {@link Task} for complete semantics.
     * 
     * @exception RetryableException The task needs to wait for some 
     *     external operation to complete or resource to become available.
     * @exception Exception The task did not complete. Abort the current workflow.
     *
     */
    void execute() throws Exception;
}
