/*
 * $Id: //depot/prod/newport_news/Radamant.1/src/java/com/netapp/common/flow/tasks/DeleteTree.java#1 $
 * 
 * Copyright (c) 2007 Network Appliance, Inc.
 * All rights reserved.
 */
package com.netapp.common.flow.tasks;

import java.io.File;

import com.netapp.common.MsgKey;
import com.netapp.common.flow.Task;
import com.netapp.common.flow.TaskInput;
import com.netapp.common.logging.NALogger;

/**
 * Recursively delete a directory. Use this with extreme caution!
 * @author sunday
 * @version $Revision: #1 $ $Date: 2010/08/20 $
 */
public class DeleteTree implements Task {
    private static final NALogger logger = NALogger.getLogger(DeleteTree.class);
    
    private String path;

    /**
     * @see com.netapp.common.flow.Task#execute()
     */
    public void execute() throws Exception {
        if (path == null) {
            return;
        }
        
        // Should we make sure this is an absolute path?
        File file = new File(path);
        if (file.exists()) {
            logger.info(MsgKey.FLOW_TASK_DELETE_TREE, file);
            deleteTree(file);
        }
        if (file.exists()) {
            throw new RuntimeException(logger.error(MsgKey.FLOW_TASK_DELETE_TREE_ERROR, file));
        }
    }

    public static void deleteTree(File file) {
        if (file.isDirectory()) {
            for (File child : file.listFiles()) {
                deleteTree(child);
            }
        }
        
        if (!file.delete()) {
            logger.warn(MsgKey.FLOW_TASK_DELETE_TREE_FILE_ERROR, file);
        }
    }
    
    @TaskInput
    public void setPath(String path) {
        this.path = path;
    }
}
