/*
 * $Id: //depot/prod/newport_news/Radamant.1/src/java/com/netapp/common/gwt/codegen/GWTAsyncInterfaceConverter.java#1 $
 * 
 * Copyright (c) 2009 NetApp, Inc.
 * All rights reserved.
 */
package com.netapp.common.gwt.codegen;

import java.util.Iterator;
import java.util.LinkedList;
import java.util.List;

import javax.lang.model.element.ExecutableElement;
import javax.lang.model.element.TypeElement;
import javax.lang.model.element.VariableElement;
import javax.lang.model.type.PrimitiveType;
import javax.lang.model.type.TypeKind;
import javax.lang.model.type.TypeMirror;
import javax.lang.model.util.ElementFilter;
import javax.lang.model.util.Elements;
import javax.lang.model.util.Types;

import com.netapp.common.generator.NetappClassGenerator;
import com.netapp.common.util.StringUtils;

public class GWTAsyncInterfaceConverter extends NetappClassGenerator {

    private TypeElement typeElement;
    private Elements elements;
    private Types types;

    public GWTAsyncInterfaceConverter(TypeElement typeElement, Elements elements, Types types) {
        super(elements.getPackageOf(typeElement).getQualifiedName().toString());
        
        this.typeElement = typeElement;
        this.elements = elements;
        this.types = types;
        
        generateClassFile();
    }

    private void generateClassFile() {
        String interfaceName = this.typeElement.getSimpleName().toString();
        
        List<String> extendedInterfaces = new LinkedList<String>();
        for (TypeMirror extendedInterfaceType : this.typeElement.getInterfaces()) {
            if (!extendedInterfaceType.toString().equals("com.google.gwt.user.client.rpc.RemoteService")) {
                extendedInterfaces.add(extendedInterfaceType.toString() + "Async");    
            }
        }
        
        addLine("public interface %sAsync %s {", 
            interfaceName,
            extendedInterfaces.isEmpty() ? ""
                : "extends " + StringUtils.collectionToString(extendedInterfaces, ", ")
        );
        increaseIndent();
        addLine();
        
        for (ExecutableElement method : ElementFilter.methodsIn(this.typeElement.getEnclosedElements())) {
            addLine("public void %s(", method.getSimpleName());
            increaseIndent();
            
            for (Iterator<? extends VariableElement> i = method.getParameters().iterator(); i.hasNext(); ) {
                VariableElement parameter = i.next();
                addLine("%s %s%s", parameter.asType(), parameter.getSimpleName(), ",");
            }

            addLine("com.google.gwt.user.client.rpc.AsyncCallback<%s> callback", 
                method.getReturnType().getKind() == TypeKind.VOID ? "Void" 
                    : method.getReturnType().getKind().isPrimitive() ? this.types.boxedClass((PrimitiveType) method.getReturnType()) : method.getReturnType()
            );
            decreaseIndent();
            
            addLine(");");
            addLine();
        }
        
        decreaseIndent();
        addLine("}");
    }

}
