/*
 * $Id: //depot/prod/newport_news/Radamant.1/src/java/com/netapp/common/logging/ReloadLoggingSignalHandlerImpl.java#1 $
 * 
 * Copyright (c) 2007 Network Appliance, Inc.
 * All rights reserved.
 */
package com.netapp.common.logging;

/*
 * CHECKSTYLE:OFF
 * There is BURT 210663 for this
 */
import java.io.IOException;
import java.net.URI;
import java.net.URISyntaxException;
import java.net.URL;
import java.util.Enumeration;

import org.apache.log4j.PropertyConfigurator;

import sun.misc.Signal;

import com.netapp.common.MsgKey;
import com.netapp.common.signal.SignalHandlerImpl;
/*
 * CHECKSTYLE:ON
 */

/**
 * Trap a signal to the java process. When handling the signal, attempt to reload
 * the Log4J property file.
 */
public class ReloadLoggingSignalHandlerImpl extends SignalHandlerImpl {
    private static final NALogger LOGGER = NALogger.getLogger(ReloadLoggingSignalHandlerImpl.class);
    private static final String LOG4J_FILE_NAME = "log4j.properties";

    public ReloadLoggingSignalHandlerImpl(String signalName, boolean isFatal, ProgramType programType) {
        super(signalName, isFatal, programType);
    }

    @Override
    public void handle(Signal sig) {
        boolean found = false;
        Enumeration<URL> resources = null;
        
        try {
            resources = this.getClass().getClassLoader().getResources(LOG4J_FILE_NAME);
        }
        catch (IOException e) {
            e.printStackTrace();
        }
        
        if (null == resources) {
            LOGGER.warn(MsgKey.SIGNAL_RELOADING_LOG4J_NO_RESOURCES, LOG4J_FILE_NAME);
        }
        
        while (resources.hasMoreElements()) {
            URL resource = resources.nextElement();

            LOGGER.debug(MsgKey.SIGNAL_RELOADING_LOG4J_LOCATED_RESOURCES, LOG4J_FILE_NAME, resource.toString());
            
            try {
                URI uri = resource.toURI();
                
                if (uri.getScheme().equals("file") && !found) {
                    PropertyConfigurator.configure(resource);
                    
                    LOGGER.info(MsgKey.SIGNAL_RELOADING_LOG4J_RELOADING_FROM_RESOURCES, resource.toString());
                    
                    found = true;
                }
            }
            catch (URISyntaxException e) {
                e.printStackTrace();
            }
        }
        
        if (!found) {
            LOGGER.warn(MsgKey.SIGNAL_RELOADING_LOG4J_NO_USABLE_RESOURCE, LOG4J_FILE_NAME);
        }
    }
}
