/*
 * $Id: //depot/prod/newport_news/Radamant.1/src/java/com/netapp/common/platform/FileSystemMount.java#1 $
 *
 * Copyright (c) 2009 NetApp, Inc.
 * All rights reserved. 
 */
package com.netapp.common.platform;

import java.io.File;

/**
 * @author sunday
 * @version $Revision: #1 $ $Date: 2010/08/20 $
 */
public class FileSystemMount {
    private final String server;
    // This is not a File; for NFS it is a remote path
    private final String device;
    private final File mountPoint;
    private final String type;
    private final String options;
    
    public FileSystemMount(String server, String device, String mountPoint, String type, String options) {
        this(server, device, new File(mountPoint), type, options);
    }
    
    public FileSystemMount(String server, String device, File mountPoint, String type, String options) {
        this.server = server;
        this.device = device;
        this.mountPoint = mountPoint;
        this.type = type;
        this.options = options;
    }
    
    /**
     * For network file systems, retrieve the remote host providing this
     * filesystem.
     * @return server providing this network filesystem, or null if the filesystem is local.
     */
    public String getServer() {
        return server;
    }

    /**
     * The device on which this filesystem is located. For network filesystems, this
     * is the remote path.
     * @return device containing this filesystem.
     */
    public String getDevice() {
        return device;
    }
    
    /**
     * The junction in the file namespace where this filesystem is mounted.
     * @return the mount point of this filesystem.
     */
    public File getMountPoint() {
        return mountPoint;
    }
    
    /**
     * The type of this filesystem.  The value is system-dependent.
     * @return filesystem type
     */
    public String getType() {
        return type;
    }
    
    /**
     * Mount options for this filesystem. Allowed values are system and filesystem dependent.
     * @return Comma separated list of mount options
     */
    public String getOptions() {
        return options;
    }
    
    public String getBackingStore() {
        return (this.server != null ? this.server + ":" : "") + this.device;
    }
    
    @Override
    public String toString() {
        return this.getBackingStore();
    }
}
