/*
 * $Id: //depot/prod/newport_news/Radamant.1/src/java/com/netapp/common/platform/aix/OperatingSystemImpl.java#1 $
 *
 * Copyright (c) 2006 Network Appliance, Inc.
 * All rights reserved.
 */
package com.netapp.common.platform.aix;

import java.io.File;
import java.util.ArrayList;
import java.util.Collection;
import java.util.List;
import java.util.regex.Matcher;
import java.util.regex.Pattern;

import com.netapp.common.MsgKey;
import com.netapp.common.execution.ShellCommand;
import com.netapp.common.execution.ShellExecutor;
import com.netapp.common.execution.ShellExecutorFactory;
import com.netapp.common.execution.ShellResult;
import com.netapp.common.execution.ShellRunnable;
import com.netapp.common.logging.NALogger;
import com.netapp.common.platform.DeviceAnalyzer;
import com.netapp.common.platform.FileSystemMount;
import com.netapp.common.platform.PartitionTable;
import com.netapp.common.platform.PlatformException;

/**
 * @author sunday
 * @version $Revision: #1 $ $Date: 2010/08/20 $
 */
public class OperatingSystemImpl extends com.netapp.common.platform.unix.OperatingSystemImpl {
    private static final NALogger logger = NALogger.getLogger(OperatingSystemImpl.class);
    
    private static final Pattern MOUNT_OUTPUT_PATTERN = 
        Pattern.compile("(\\S+)?\\s+(\\S+)\\s+(\\S+)\\s+(\\S+)\\s+(\\w+ \\d+ \\d+:\\d+)\\s+(\\S*)\\s*");
    
    private static final int MOUNT_OUTPUT_SERVER_GROUP = 1;
    private static final int MOUNT_OUTPUT_DEVICE_GROUP = 2;
    private static final int MOUNT_OUTPUT_MOUNT_POINT_GROUP = 3;
    private static final int MOUNT_OUTPUT_TYPE_GROUP = 4;
    @SuppressWarnings("unused")
    private static final int MOUNT_OUTPUT_DATE_GROUP = 5;
    private static final int MOUNT_OUTPUT_OPTIONS_GROUP = 6;
    

    /**
     *
     */
    public OperatingSystemImpl() {
        super();
    }

    @Override
    public String getLibraryEnvironmentName() {
        return "LIBPATH";
    }

    @Override
    public DeviceAnalyzer createDeviceAnalyzer() {
        return new AIXDeviceAnalyzer();
    }

    @Override
    protected ShellRunnable createDFCommand(List<File> fileList) {
        ShellCommand shellCommand = new ShellCommand();
        shellCommand.append("df", "-P");
        for (File file : fileList) {
            shellCommand.append(file.getAbsolutePath());
        }
        return shellCommand;
    }

    @Override
    protected DFResultConstructor createDFResultConstructor() {
        return new com.netapp.common.platform.aix.DFResultConstructor();
    }

    @Override
    public PartitionTable readPartitionTable(File device) {
        throw new UnsupportedOperationException();
    }

    @Override
    public Collection<FileSystemMount> getMountedFilesystems() {
        ShellExecutor executor = ShellExecutorFactory.createShellExecutor();
        ShellResult result = executor.execute(getMountTableCommand(), 60 * 1000);
        if (result.isNonZeroExitCode()) {
            throw new PlatformException(
                logger.debug(MsgKey.PLATFORM_MOUNT_TABLE_READ_FAILED, result.allOutAsString()));
        }
        List<FileSystemMount> results = new ArrayList<FileSystemMount>();
        for (String line : result.getAllOut()) {
            Matcher m = MOUNT_OUTPUT_PATTERN.matcher(line);
            // Don't list autofs synthetic mounts
            if (m.matches() 
                    && !"autofs".equals(m.group(MOUNT_OUTPUT_TYPE_GROUP))) {
                results.add(new FileSystemMount(
                    m.group(MOUNT_OUTPUT_SERVER_GROUP),
                    m.group(MOUNT_OUTPUT_DEVICE_GROUP),
                    m.group(MOUNT_OUTPUT_MOUNT_POINT_GROUP),
                    m.group(MOUNT_OUTPUT_TYPE_GROUP),
                    m.group(MOUNT_OUTPUT_OPTIONS_GROUP)));          
            }
        }
        return results;
    }
    
    /**
     * Unit test hook
     */
    protected ShellCommand getMountTableCommand() {
        return new ShellCommand("/usr/sbin/mount");
    }
}
