/*
 * $Id: //depot/prod/newport_news/Radamant.1/src/java/com/netapp/common/platform/linux/DFResultConstructor.java#1 $
 * 
 * Copyright (c) 2005 Network Appliance, Inc.
 * All rights reserved.
 */
package com.netapp.common.platform.linux;

import java.io.File;
import java.util.List;
import java.util.regex.Matcher;
import java.util.regex.Pattern;

import com.netapp.common.MsgKey;
import com.netapp.common.execution.ShellResult;
import com.netapp.common.logging.NALogger;
import com.netapp.common.platform.PlatformException;
import com.netapp.common.platform.unix.DFResult;
import com.netapp.common.platform.unix.DFResult.FileInfo;

/**
 * Parses the result of the 'df' command on Linux.  
 * 
 * @author sirianni
 * @version $Revision: #1 $ $Date: 2010/08/20 $
 */
public class DFResultConstructor extends com.netapp.common.platform.unix.DFResultConstructor {

    private static final NALogger logger = NALogger.getLogger(DFResultConstructor.class);
    
    /**
     * Matches 'df -PT' output on Linux:
     * "weever:/vol/eric/test nfs     81920      1408     80512       2% /mnt/eric_test"
     * Collects the first, second, and seventh non-whitespace chunks into groups
     */
    private static final Pattern PATTERN = Pattern.compile("(\\S+)\\s+(\\S+)\\s+\\S+\\s+\\S+\\s+\\S+\\s+\\S+\\s+(\\S+)\\s*");
    
    DFResultConstructor() {
        super(1, 1);
    }
    
    @Override
    protected DFResult doConstruct(List<File> fileList, ShellResult shellResult) {
        DFResult dfResult = new DFResult();
        int outputLineIndex = 1;
        for (File file : fileList) {
            String dfLine = shellResult.getStdOut().get(outputLineIndex++);
                
            Matcher matcher = PATTERN.matcher(dfLine);
                
            if (!matcher.matches()) {
                String error = logger.error(MsgKey.PLATFORM_ERROR_PARSING_D_F_OUTPUT, shellResult.getStdOut().toString());
                throw new PlatformException(error);
            }
    
            String fileSystem = matcher.group(1);
            String mountPoint = matcher.group(3);
            
            String fsTypeString = matcher.group(2);
            String filer = null;
            String export = null;
            
            if (fsTypeString.equalsIgnoreCase("nfs")) {
                String[] split = fileSystem.split(":");
                if (split.length != 2) {
                    String error = logger.error(MsgKey.PLATFORM_ERROR_PARSING_D_F_OUTPUT, shellResult.getStdOut().toString());
                    throw new PlatformException(error);
                }
                filer = split[0];
                export = split[1];
            }
            
            dfResult.addFileInfo(file, new FileInfo(fileSystem, new File(mountPoint), filer, export));
        }
        return dfResult;
    }
}
