/*
 * $Id: //depot/prod/newport_news/main/src/java/com/netapp/common/platform/linux/SLES10SP2MultipathHandler.java#2 $
 * 
 * Copyright (c) 2008 NetApp, Inc.
 * All rights reserved.
 */
package com.netapp.common.platform.linux;

import java.io.File;
import java.util.List;
import java.util.regex.Matcher;
import java.util.regex.Pattern;

import com.netapp.common.execution.ShellCommand;
import com.netapp.common.execution.ShellResult;

/**
 * Manages device-mapper-multipath devices from SLES 10 SP2
 * @author sunday
 * @version $Revision: #2 $ $Date: 2008/05/29 $
 */
public class SLES10MultipathHandler extends RH5MultipathHandler {
    
    private static final Pattern DMSETUP_ENTRY_PATTERN = Pattern.compile("(.*):(\\d+):(\\d+)");
    private static final int DEVICE_PARTITION_GROUP = 1;
    
    public SLES10MultipathHandler(OperatingSystemImpl operatingSystem) {
        super(operatingSystem);
    }

    /**
     * @see com.netapp.common.platform.linux.BlockDeviceHandler#deviceFromMajorMinor(int, int)
     */
    public File deviceFromMajorMinor(int major, int minor) {
        // Strictly speaking, we don't need the major and minor, but it's easier
        // to differentiate from an error condition if we have a little more to match
        // on than the "name" column itself.
        // This can be done with JNI against the device nodes in /dev/mapper too.
        ShellCommand cmd = new ShellCommand("/sbin/dmsetup", "info", 
            "--noheadings",
            "-c", 
            "-o", "major,minor,name",
            "-j", Integer.toString(major), 
            "-m", Integer.toString(minor));
        ShellResult dmsetupResult = getShellExecutor().execute(cmd);
        final List<String> stdOutput = dmsetupResult.getStdOut();
        for (String entry : stdOutput) {
            Matcher matcher = DMSETUP_ENTRY_PATTERN.matcher(entry);
            System.out.println("Entry string" + entry);
            if (matcher.matches()) {
                return new File("/dev/mapper/" + matcher.group(DEVICE_PARTITION_GROUP));
            }
        }
        return null;
    }
}
