/*
 * $Id: //depot/prod/newport_news/Radamant.1/src/java/com/netapp/common/platform/linux/smbios/SystemInformationHeader.java#1 $
 * 
 * Copyright (c) 2009 NetApp, Inc.
 * All rights reserved.
 */
package com.netapp.common.platform.linux.smbios;

import java.io.IOException;
import java.io.RandomAccessFile;

/**
 * After the BasicStructuredHeader, the system information header includes the
 * following fields:
 * 
 * <ul>
 * <li>BYTE: manufacturer
 * <li>BYTE: product name
 * <li>BYTE: version
 * <li>BYTE: serial number
 * <li>UuidHeader
 * </ul>
 * 
 * @author josh
 * 
 */
final class SystemInformationHeader {
    private static final int HEADER_BEFORE_UUID_IN_BYTES = 4;

    private UuidHeader       uuidHeader;

    protected SystemInformationHeader() {
    }

    public static SystemInformationHeader read(RandomAccessFile file) throws IOException {
        // skip to the UuidHeader
        byte [] skip = new byte[HEADER_BEFORE_UUID_IN_BYTES];
        file.read(skip);

        SystemInformationHeader header = new SystemInformationHeader();
        header.setUuidHeader(UuidHeader.read(file));

        return header;
    }

    public String getUuid() {
        return getUuidHeader().toString();
    }

    protected UuidHeader getUuidHeader() {
        return uuidHeader;
    }

    protected void setUuidHeader(UuidHeader uuidHeader) {
        this.uuidHeader = uuidHeader;
    }
}
