/*
 * $Id: //depot/prod/newport_news/Radamant.1/src/java/com/netapp/common/platform/sunos/DFResultConstructor.java#1 $
 * 
 * Copyright (c) 2005 Network Appliance, Inc.
 * All rights reserved.
 */
package com.netapp.common.platform.sunos;

import java.io.File;
import java.util.List;
import java.util.regex.Matcher;
import java.util.regex.Pattern;

import com.netapp.common.MsgKey;
import com.netapp.common.execution.ShellResult;
import com.netapp.common.logging.NALogger;
import com.netapp.common.platform.PlatformException;
import com.netapp.common.platform.unix.DFResult;
import com.netapp.common.platform.unix.DFResult.FileInfo;

/**
 * Parses the result of the 'df' command on Solaris.  
 * 
 * @author sirianni
 * @version $Revision: #1 $ $Date: 2010/08/20 $
 */
public class DFResultConstructor extends com.netapp.common.platform.unix.DFResultConstructor {

    private static final NALogger logger = NALogger.getLogger(DFResultConstructor.class);
    
    /**
     * Matches first line of 'df -g' output on Solaris:
     * "/mnt/eric_test     (weever:/vol/eric/test):      8192 block size           512 frag size"
     * Designed to extract the "/mnt/eric_test" and "(weever:/vol/eric/test)" portions.
     */
    private static final Pattern LINE_1_PATTERN = Pattern.compile("([^\\(]+)\\(([^\\)]+).*");
    
    /**
     * Matches last line of 'df -g' output on Solaris:
     * "     nfs fstype         00000000 flag        4294967295 filename length"
     * Designed to extract the "nfs" portion (the fstype).
     */
    private static final Pattern LINE_4_PATTERN = Pattern.compile("\\s*(\\S+)\\s*fstype.*");

    DFResultConstructor() {
        super(0, 5);
    }
    
    @Override
    protected DFResult doConstruct(List<File> fileList, ShellResult shellResult) {
        DFResult dfResult = new DFResult();
        int outputLineIndex = 0;
        for (File file : fileList) {
            String dfLine1 = shellResult.getStdOut().get(outputLineIndex);
                
            Matcher line1Matcher = LINE_1_PATTERN.matcher(dfLine1);
            
            if (!line1Matcher.matches()) {
                String error = logger.error(MsgKey.PLATFORM_ERROR_PARSING_D_F_OUTPUT, shellResult.getStdOut().toString());
                throw new PlatformException(error);
            }
            
            String fileSystem = line1Matcher.group(2).trim();
            String mountPoint = line1Matcher.group(1).trim();
            
            String filer = null;
            String export = null;
            
            String dfLine4 = shellResult.getStdOut().get(outputLineIndex + 3);
            
            Matcher line4Matcher = LINE_4_PATTERN.matcher(dfLine4);
            
            if (!line4Matcher.matches()) {
                String error = logger.error(MsgKey.PLATFORM_ERROR_PARSING_D_F_OUTPUT, shellResult.getStdOut().toString());
                throw new PlatformException(error);
            }
    
            String fsTypeString = line4Matcher.group(1).trim();
            if (fsTypeString.equalsIgnoreCase("nfs")) {
                String[] split = fileSystem.split(":");
                if (split.length != 2) {
                    String error = logger.error(MsgKey.PLATFORM_ERROR_PARSING_D_F_OUTPUT, shellResult.getStdOut().toString());
                    throw new PlatformException(error);
                }
                filer = split[0];
                export = split[1];
            }
            
            dfResult.addFileInfo(file, new FileInfo(fileSystem, new File(mountPoint), filer, export));
            
            outputLineIndex += 5;
        }
        return dfResult;
    }
}
