/*
 * $Id: //depot/prod/newport_news/Radamant.1/src/java/com/netapp/common/platform/unix/UnixAuthenticatorNativeImpl.java#1 $
 *
 * Copyright (c) 2008 Network Appliance, Inc.
 * All rights reserved.
 *
 */
package com.netapp.common.platform.unix;

import com.netapp.common.logging.NALogger;
import com.netapp.common.platform.Authenticator;
import com.netapp.common.util.ConfigBundle;
import com.netapp.common.util.ConfigProperty;

/**
 * Authenticates using:
 *     1.) PAM under Linux, Solaris, HPUX and OSX
 *     2.) LAM under AIX
 * @author rippy
 * @version $Revision: #1 $ $Date: 2010/08/20 $
 */
public class UnixAuthenticatorNativeImpl implements Authenticator {

    private static NALogger logger = NALogger.getLogger();

    native void nativeAuthenticate(String appname, String username, String password);

    static {
        System.loadLibrary("com_netapp_common_platform_unix_UnixAuthenticatorNativeImpl");
    }

    private static final ConfigProperty<String> PAM_SERVICE_NAME = ConfigProperty.stringProperty("pam.service.name",
        "snapmanager");

    protected String errorMessage;

    protected ConfigBundle configBundle;

    protected String serviceName;

    public UnixAuthenticatorNativeImpl(ConfigBundle configBundle) {
        this.configBundle = configBundle;
        this.serviceName = PAM_SERVICE_NAME.lookup(this.configBundle);
    }

    public void logInternal(String msg) {
        logger.internal(msg);
    }

    public String getErrorMessage() {
        return this.errorMessage;
    }

    public void setErrorMessage(String msg) {
        if (msg == null) {
            this.errorMessage = null;
        }
        else {
            StringBuffer sb = new StringBuffer(msg);
            this.errorMessage = sb.toString();
        }
    }

    public boolean authenticate(String username, String password) {
        setErrorMessage(null);
        
        this.nativeAuthenticate(this.serviceName, username, password);
        final boolean authenticated = this.errorMessage == null;
        if (authenticated) {
            logger.internal("authenticate: User '%s' granted access.", username);
        }
        else {
            logger.internal("authenticate: User '%s' NOT granted access", username);
        }
        return authenticated;
    }

    public static void main(String[] args) {
        if (args.length < 2) {
            System.out.println("UnixAuthenticatorNativeImpl syntax:\n\n\t<username> <password>");
            System.exit(1);
        }
        
        String username = args[0];
        String password = args[1];
        UnixAuthenticatorNativeImpl authenticator = new UnixAuthenticatorNativeImpl(new ConfigBundle());
        System.out.println("Authenticated ['" + authenticator.authenticate(username, password) + "']\terrorString['" + authenticator.getErrorMessage() + "']");
    }
}
