/*
 * $Id: //depot/prod/newport_news/Radamant.1/src/java/com/netapp/common/util/ChangeSet.java#1 $
 * 
 * Copyright (c) 2008 NetApp, Inc.
 * All rights reserved.
 */
package com.netapp.common.util;

import java.util.List;

/**
 * A collection of state change notifications. Each collection also has a tag representing
 * generation number of the newest entry, using this tag in a subsequent call to 
 * {@link com.netapp.common.util.ChangeTracker#getChanges(long)} will return the next set of
 * changes newer than this set.
 * 
 * @author sunday
 * @version $Revision: #1 $ $Date: 2010/08/20 $
 */
public class ChangeSet<T> {
    private long tag;
    private List<T> changes;
    
    public ChangeSet(long tag, List<T> changes) {
        this.tag = tag;
        this.changes = changes;
    }
    
    /**
     * The generation number of the newest member of this set.
     * @return
     */
    public long getTag() {
        return this.tag;
    }
    
    /**
     * The list of change notifications for this set. Entries are always
     * in order with the oldest changes first.
     * @return
     */
    public List<T> getChanges() {
        return this.changes;
    }
}
