/*
 * $Id: //depot/prod/newport_news/Radamant.1/src/java/com/netapp/common/util/Pid.java#1 $
 * 
 * Copyright (c) 2007 Network Appliance, Inc.
 * All rights reserved.
 */
package com.netapp.common.util;

import java.lang.management.ManagementFactory;
import java.lang.management.RuntimeMXBean;
import java.util.regex.Matcher;
import java.util.regex.Pattern;

/**
 * Use a MX Bean interface to determine the pid of the running process. This is being used in an attempt to avoid
 * using native calls.
 * 
 * @author joshb
 *
 */
public class Pid {
    private int value;
    
    public Pid() {
        setValue(getPidFromMXBean());
    }
    
    public int getValue() {
        return value;
    }
    
    protected void setValue(int value) {
        this.value = value;
    }

    /**
     * Determine the PID from the RuntimeMXBean's name. The name has the following format on several hosts that were tested.
     * 
     * <pre>
     *   [0-9]+@.*
     * </pre>
     * 
     * Which means a number of varying length, followed by a '@' following by anything else. The number at the front is the PID.
     * 
     * If the pid cannot be found, a -1 is returned
     * 
     * @return process pid, or -1 if unable to determine
     */
    protected int getPidFromMXBean() {
        RuntimeMXBean rtb = ManagementFactory.getRuntimeMXBean();
        
        Pattern pattern = Pattern.compile("([0-9]+)@.*");
        Matcher matcher = pattern.matcher(rtb.getName());
        
        if (matcher.matches()) {
            return Integer.parseInt(matcher.group(1));
        }
        
        return -1;
    }

}
