/*
 * $Id: //depot/prod/newport_news/Radamant.1/src/java/com/netapp/common/util/Size.java#1 $
 * 
 * Copyright (c) 2008 Network Appliance, Inc.
 * All rights reserved.
 */
package com.netapp.common.util;

import java.util.regex.Matcher;
import java.util.regex.Pattern;

import org.apache.commons.lang.builder.HashCodeBuilder;

public class Size implements Comparable<Size> {

    private long value;
    
    private Unit unit;

    public Size(long value, Unit unit) {
        this.value = value;
        this.unit = unit;
    }

    public long getValue() {
        return this.value;
    }
    
    public Unit getUnit() {
        return this.unit;
    }
    
    public Size convertTo(Unit destUnit) {
        return new Size(this.valueIn(destUnit), destUnit);
    }
    
    public long valueIn(Unit destUnit) {
        return (this.value * this.unit.multiplier) / destUnit.multiplier;
    }
    
    @Override
    public boolean equals(Object object) {
        if (!(object instanceof Size)) {
            return false;
        }
        
        Size rhs = (Size) object;
        return this.value == rhs.valueIn(this.unit);
    }
    
    @Override
    public int hashCode() {
        return new HashCodeBuilder()
            .append(this.valueIn(Size.Unit.B))
            .toHashCode();
    }
    
    @Override
    public String toString() {
        return String.format("%,d%s", this.value, this.unit);
    }
    
    public enum Unit {
        B(1), 
        KB(1024), 
        MB(1024 * 1024), 
        GB(1024 * 1024 * 1024), 
        TB(1024 * 1024 * 1024 * 1024);
        
        private long multiplier;

        Unit(long multiplier) {
            this.multiplier = multiplier;
        }
    }

    public int compareTo(Size other) {
        return (int) (this.valueIn(Unit.B) - other.valueIn(Unit.B)); 
    }

    private static final Pattern SIZE_STRING_PATTERN = Pattern.compile("([0-9,]+)([a-zA-Z]+)"); 
    
    public static Size fromString(String string) {
        Matcher matcher = SIZE_STRING_PATTERN.matcher(string);
        
        if (!matcher.matches()) {
            throw new IllegalArgumentException(string);
        }
        
        return new Size(
            Long.parseLong(matcher.group(1).replace(",", "")),
            Unit.valueOf(matcher.group(2).toUpperCase())
        );
    }
}
