/*
 * $Id: //depot/prod/newport_news/Radamant.1/src/java/com/netapp/common/zapi/AbstractZAPIExecutor.java#1 $
 * 
 * Copyright (c) 2007 Network Appliance, Inc.
 * All rights reserved.
 */
package com.netapp.common.zapi;

import java.io.IOException;

import netapp.manage.NaAPIFailedException;
import netapp.manage.NaException;
import netapp.manage.NaServer;

import com.netapp.common.MsgKey;
import com.netapp.common.logging.LogLevel;

/**
 * Each ZAPI executor must create a server instance and then execute the
 * ZAPI command. All of the plumbing work is put in place in this abstract
 * class. It is the responsibility of each implementation to set the filer
 * and NaServer objects when constructing the executor.
 * 
 * @author joshb
 *
 */
public abstract class AbstractZAPIExecutor implements ZAPIExecutor {
    /** Hostname or IP address of the filer to execute against */
    protected String filer;

    /** Manage OnTAP server object */
    protected NaServer naServer;

    private boolean quietOnError;
    
    public AbstractZAPIExecutor(String filer) {
        this.filer = filer;
    }
    
    /**
     * @throws ZAPIExecuteException 
     * @see com.netapp.common.zapi.ZAPIExecutor#execute(com.netapp.common.zapi.ZAPICommand)
     */
    public ZAPIResult execute(ZAPICommand command) throws ZAPIExecuteException {
        Exception exception;
        try {
            ZAPIExecutor.logger.debug(MsgKey.ZAPI_EXECUTOR_ZAPI_REQUEST, 
                command.getCommandName(),
                this.filer,
                command.toString());
            
            ZAPIResult result = new ZAPIResult(
                getNaServer().invokeElem(command.getRequest()));
            
            ZAPIExecutor.logger.debug(MsgKey.ZAPI_EXECUTOR_ZAPI_RESPONSE, 
                command.getCommandName(),
                this.filer,
                result.toString());
       
            return result;
        }
        catch (NaAPIFailedException e) {
            // The API Call itself returned an error. We handle this one differently
            // so callers can decide if it's an issue or not.
            ZAPIExecutor.logger.debug(MsgKey.ZAPI_EXECUTOR_API_RETURNED_FAILURE, 
                command.getCommandName(),
                this.filer,
                e.getReason(), 
                e.getErrno());
            throw new ZAPIExecuteException(e);
        }
        catch (NaException e) {
            exception = e;
        }
        catch (IOException e) {
            exception = e;
        }

        // An unexpected error happened.
        // This one is not declared so the caller doesn't have to
        // handle it.
        throw new ZAPIRunnerException(
            ZAPIExecutor.logger.log(this.quietOnError ? LogLevel.DEBUG : LogLevel.ERROR,
                MsgKey.ZAPI_EXECUTOR_EXCEPTION_DURING_EXECUTE, 
                exception,
                command.getCommandName(),
                this.filer,
                exception.getMessage()), 
                exception);
    }

    protected String getFiler() {
        return this.filer;
    }

    protected void setFiler(String filer) {
        this.filer = filer;
    }

    protected NaServer getNaServer() {
        return this.naServer;
    }

    protected void setNaServer(NaServer naServer) {
        this.naServer = naServer;
    }
    
    public void setAdminUser(String username, String password) {
        getNaServer().setAdminUser(username, password);
    }
    
    public void setQuietOnError(boolean quietOnError) {
        this.quietOnError = quietOnError;
    }
}
