/*
 * $Id: //depot/prod/newport_news/Radamant.1/src/java/com/netapp/common/zapi/DfmZAPIExecutor.java#1 $
 * 
 * Copyright (c) 2007 Network Appliance, Inc.
 * All rights reserved.
 */
package com.netapp.common.zapi;

import java.io.IOException;
import java.net.UnknownHostException;

import com.netapp.common.MsgKey;

import netapp.manage.NaServer;

/**
 * Construct a ZAPI executor which will communicate with a DFM server.
 * @author joshb
 *
 */
public class DfmZAPIExecutor extends AbstractZAPIExecutor {
    protected static final int DFM_API_MAJOR_VERSION = 1;
    protected static final int DFM_API_MINOR_VERISION = 0;
    public static final int DFM_ZAPI_PORT = 8488;

    public DfmZAPIExecutor(String filer) throws InvalidTargetException {
        this(filer, DFM_ZAPI_PORT, "", "");
    }
    
    public DfmZAPIExecutor(String filer, int port) throws InvalidTargetException {
        this(filer, port, "", "");
    }
    
    public DfmZAPIExecutor(String filer, String username, String password) throws InvalidTargetException {
        this(filer, DFM_ZAPI_PORT, username, password);
    }
    
    public DfmZAPIExecutor(String filer, int port, String username, String password) throws InvalidTargetException {
        super(filer);
        try {
            setNaServer(new NaServer(filer, DFM_API_MAJOR_VERSION, DFM_API_MINOR_VERISION));
            getNaServer().setKeepAliveEnabled(true);
            getNaServer().setStyle(NaServer.STYLE_LOGIN_PASSWORD);
            getNaServer().setTransportType(NaServer.TRANSPORT_TYPE_HTTPS);        
            getNaServer().setServerType(NaServer.SERVER_TYPE_DFM);
            getNaServer().setPort((port > 0) ? port : DFM_ZAPI_PORT);
            setAdminUser(username, password);
        }
        catch (UnknownHostException e) {
            throw new InvalidTargetException(
                logger.error(MsgKey.ZAPI_EXECUTOR_UNKNOWN_DFM_TARGET, e, filer));
        }
    }

    @Override
    public ZAPIResult execute(ZAPICommand command) throws ZAPIExecuteException {
        try {
            return super.execute(command);
        } 
        catch (ZAPIRunnerException e) {
            if (e.getCause() instanceof IOException) {
                throw new ZAPIRunnerException(logger.error(MsgKey.ZAPI_EXECUTOR_DFM_DOWN, 
                    e,
                    this.filer,
                    e.getCause().getMessage()), 
                    e);
            }
            
            throw e;
        }
    }
    
    
}
