/*
 * $Id: //depot/prod/newport_news/Radamant.1/src/java/com/netapp/common/zapi/DirectZAPIExecutor.java#1 $
 * 
 * Copyright (c) 2007 Network Appliance, Inc.
 * All rights reserved.
 */
package com.netapp.common.zapi;

import java.net.UnknownHostException;

import netapp.manage.NaServer;

import com.netapp.common.MsgKey;
import com.netapp.common.logging.NALogger;

/**
 * Basic executor to execute a {@link ZAPICommand} directly against a storage system
 * and return a {@link ZAPIResult}.
 * @version $Revision: #1 $ $Date: 2010/08/20 $
 */
public class DirectZAPIExecutor extends AbstractZAPIExecutor {
    private static final NALogger logger = NALogger.getLogger(DirectZAPIExecutor.class);
    
    protected static final String DEFAULT_FILER_USERNAME = "root";
    protected static final String DEFAULT_FILER_PASSWORD = "";
    protected static final int DEFAULT_FILER_HTTPS_PORT = 443;
    protected static final int DEFAULT_FILER_HTTP_PORT = 80;
    
    /** Major version of the zapi interface we wish to call. */
    protected int agentMajorVersion = 1;
    
    /** 
     * Minor version of the zapi interface we wish to call.
     * 1.2 is the earliest known version corresponding to 6.4.1
     * 
     */
    protected int agentMinorVersion = 2;
    
    /**
     * Construct a new executor against the specified filer, assuming
     * the root user with no password.
     * @param filer Filer to execute zapi commands against.
     * @throws InvalidTargetException 
     * @throws UnknownHostException 
     */
    public DirectZAPIExecutor(String filer) throws InvalidTargetException {
        this(filer, DEFAULT_FILER_HTTPS_PORT, DEFAULT_FILER_USERNAME, DEFAULT_FILER_PASSWORD, true);

    }
    
    public DirectZAPIExecutor(String filer, int port) throws InvalidTargetException {
        this(filer, port, DEFAULT_FILER_USERNAME, DEFAULT_FILER_PASSWORD, true);
    }
    
    public DirectZAPIExecutor(String filer, String username, String password) throws InvalidTargetException {
        this(filer, DEFAULT_FILER_HTTPS_PORT, username, password, true);
    }
    
    public DirectZAPIExecutor(String filer, String username, String password, boolean ssl) throws InvalidTargetException {
        this(filer, ssl ? DEFAULT_FILER_HTTPS_PORT : DEFAULT_FILER_HTTP_PORT, username, password, ssl);
    }
    
    public DirectZAPIExecutor(String filer, int port, String username, String password) throws InvalidTargetException {
        this(filer, port, username, password, true);
    }
    
    public DirectZAPIExecutor(String filer, int port, String username, String password, boolean ssl) throws InvalidTargetException {
        super(filer);
        try {
            setNaServer(new NaServer(filer, this.agentMajorVersion, this.agentMinorVersion));
            getNaServer().setKeepAliveEnabled(true);
            getNaServer().setStyle(NaServer.STYLE_LOGIN_PASSWORD);
            getNaServer().setServerType(NaServer.SERVER_TYPE_FILER);
            getNaServer().setTransportType(ssl ? NaServer.TRANSPORT_TYPE_HTTPS : NaServer.TRANSPORT_TYPE_HTTP);
            getNaServer().setPort((port > 0) ? port : (ssl ? DEFAULT_FILER_HTTPS_PORT : DEFAULT_FILER_HTTP_PORT));
            setAdminUser(username, password);
        }
        catch (UnknownHostException e) {
            throw new InvalidTargetException(
                logger.error(MsgKey.ZAPI_EXECUTOR_UNKNOWN_STORAGE_TARGET, e, filer));
        }
    }
}
