/*
 * $Id: //depot/prod/newport_news/Radamant.1/src/java/com/netapp/common/zapi/generator/autozapi/AutoZapiNamespaceGenerator.java#1 $
 * 
 * Copyright (c) 2009 Network Appliance, Inc.
 * All rights reserved.
 */
package com.netapp.common.zapi.generator.autozapi;

import java.io.File;
import java.io.IOException;
import java.util.LinkedList;
import java.util.Queue;
import java.util.Set;
import java.util.TreeSet;

import com.netapp.common.MsgKey;
import com.netapp.common.generator.NetappClassGenerator;
import com.netapp.common.logging.NALogger;
import com.netapp.common.util.ClassUtils;
import com.netapp.common.zapi.generator.GeneratorUtils;
import com.netapp.common.zapi.generator.NamespaceGenerator;
import com.netapp.common.zapi.parser.Api;
import com.netapp.common.zapi.parser.Category;
import com.netapp.common.zapi.parser.Namespace;
import com.netapp.common.zapi.parser.Type;

public class AutoZapiNamespaceGenerator extends NamespaceGenerator {
    
    private static final NALogger logger = NALogger.getLogger(AutoZapiNamespaceGenerator.class);

    public AutoZapiNamespaceGenerator(AutoZapiGeneratorFactory generatorFactory, Namespace namespace) {
        super(generatorFactory, namespace);
    }
    
    @Override
    public String getNameGenerated() {
        return GeneratorUtils.removeDashes(getTarget().getName());
    }
    
    public String getPackageName() {
        String packageName = ((AutoZapiGeneratorFactory) getGeneratorFactory()).getPackageName(); 
        return (packageName != null
                ? packageName + "."
                : "") + getNameGenerated();
    }
    
    /**
     * Generate class files for all types in this namespace
     * @param basePackageName Base java package name. The current namespace will be appended to this.
     * @param baseDir Base classpath directory to generate into. The package name will be converted to a file path and appended to this.
     * @throws IOException
     */
    @SuppressWarnings("unchecked")
    public void generateClassFiles(File baseDir) throws IOException {
        logger.info(MsgKey.ZAPI_GENERATOR_WRITING_NAMESPACE, getTarget().getName());
        
        Set<Type> selectedTypes = new TreeSet<Type>();
        Queue<Type> workQueue = new LinkedList<Type>();
        
        Set<Category> categories = new TreeSet<Category>();
        
        for (Type type : getTarget().getTypes()) {
            if (type instanceof Api) {
                selectedTypes.add(type);
            }
        }
        
        workQueue.addAll(selectedTypes);
        
        Type referent = workQueue.poll();
        while (referent != null) {
            for (Type referee : referent.getReferencedTypes()) {
                if (!selectedTypes.contains(referee)) {
                    selectedTypes.add(referee);
                    workQueue.add(referee);
                }
            }
            referent = workQueue.poll();
        }
        
        for (Type type : selectedTypes) {
            AutoZapiTypeGenerator typeGenerator = (AutoZapiTypeGenerator) getGeneratorFactory().getTypeGenerator(type);
            File outputDir = new File(baseDir, ClassUtils.classNameToFilePath(typeGenerator.getPackageName()));
            if (!outputDir.exists() && !outputDir.mkdirs()) {
                throw new IOException("Unable to create package directory " + outputDir);
            }
            
            typeGenerator.writeCode(outputDir.getPath());
            categories.add(type.getCategory());
        }
        
        for (Category cat : categories) {
            AutoZapiCategoryGenerator categoryGenerator = (AutoZapiCategoryGenerator) getGeneratorFactory().getCategoryGenerator(cat);
            String packageName = this.getPackageName() + "." + categoryGenerator.getNameGenerated();
            CategoryInfoGenerator catGenerator = new CategoryInfoGenerator(packageName, cat);
            File outputDir = new File(baseDir, ClassUtils.classNameToFilePath(packageName));
            catGenerator.writeCode(outputDir.getPath(), "package-info.java");
        }
    }
    
    private class CategoryInfoGenerator extends NetappClassGenerator {
        CategoryInfoGenerator(String packageName, Category category) {
            super(packageName, AutoZapiGenerator.FORMATTER);
            
            AutoZapiCategoryGenerator categoryGenerator = (AutoZapiCategoryGenerator) getGeneratorFactory().getCategoryGenerator(category);
            
            addPackageJavadoc(categoryGenerator.getSynopsisGenerated());

            if (category.getDiscussion() != null) {
                addPackageJavadoc("<hr>");
                addPackageJavadoc("<b>Discussion</b>: ");
                addPackageJavadoc(categoryGenerator.getDiscussionGenerated());
            }
        }
    }
}
